-- scope stuff
qualityoflife = qualityoflife or {};
local q = qualityoflife;

q.modules = q.modules or {};
q.modules["QoL_ClassicHeals-TBC"] = q.modules["QoL_ClassicHeals-TBC"] or {};
local c = q.modules["QoL_ClassicHeals-TBC"];

local function ContainsFrame(collection, frame)
    for _, existingFrame in ipairs(collection) do
        if existingFrame:GetName() == frame:GetName() then
            return true;
        end
    end
end

local function GetFrame(frameName)
    local frame = _G[frameName];
    if frame and frame:IsVisible() then
        return frame;
    end
end

local function AddFrame(collection, frame)
    if frame and not ContainsFrame(collection, frame) then
        tinsert(collection, frame);
    end
end

local function AddFrames(collection, frames)
    for _, frame in ipairs(frames) do
        AddFrame(collection, frame);
    end
end

local function AddFrameName(collection, frameName)
    AddFrame(collection, GetFrame(frameName));
end

local function AddFrameNames(collection, frameNames)
    for _, frameName in ipairs(frameNames) do
        AddFrame(collection, GetFrame(frameName));
    end
end

local function GetRaidFrames(target)
    local frames = {};

    for n = 1, 10 do -- pets increase the number of frames (not confirmed)
        local compactPartyFrameName = "CompactPartyFrameMember" .. n;
        local compactPartyFrame = _G[compactPartyFrameName];
        if compactPartyFrame and compactPartyFrame:IsVisible() and _G[compactPartyFrameName .. "Name"] and
            _G[compactPartyFrameName .. "Name"]:GetText() == target then
            tinsert(frames, compactPartyFrame);
        end
    end

    for n = 1, 80 do -- pets increase the number of frames
        local compactRaidFrameName = "CompactRaidFrame" .. n;
        local compactRaidFrame = _G[compactRaidFrameName];
        if compactRaidFrame and compactRaidFrame:IsVisible() and _G[compactRaidFrameName .. "Name"] and
            _G[compactRaidFrameName .. "Name"]:GetText() == target then
            tinsert(frames, compactRaidFrame);
        end
    end

    for groupId = 1, 8 do
        for subId = 1, 5 do
            local compactRaidGroupFrameName = "CompactRaidGroup" .. groupId .. "Member" .. subId;
            local compactRaidGroupFrame = _G[compactRaidGroupFrameName];
            if compactRaidGroupFrame and compactRaidGroupFrame:IsVisible() and _G[compactRaidGroupFrameName .. "Name"] and
                _G[compactRaidGroupFrameName .. "Name"]:GetText() == target then
                tinsert(frames, compactRaidGroupFrame);
            end
        end
    end

    return frames;
end

local function GetRaidPulloutFrame(target)
    for groupId = 1, 8 do
        for subId = 1, 5 do
            local raidPulloutName = "RaidPullout" .. groupId .. "Button" .. subId;
            if _G[raidPulloutName] and _G[raidPulloutName .. "Name"] and _G[raidPulloutName .. "Name"]:GetText() ==
                target then
                return _G[raidPulloutName];
            end
        end
    end
end

-- addons / custom UIs

local function GetElvUiPartyFrameNames(target)
    local result = {};
    if ElvUI then
        local unitId = c:GetAffectedPartyUnitId(target);
        if not unitId and target == GetUnitName("player") then
            unitId = "player";
        end

        if unitId then
            for partyId = 1, 5 do
                local frameName = "ElvUF_PartyGroup1UnitButton" .. partyId;
                if _G[frameName] and _G[frameName].unit and _G[frameName].unit == unitId then
                    tinsert(result, frameName);
                end
            end
        end
    end
    return result;
end

local function GetElvUiRaidFrameNames(target)
    local result = {};
    if ElvUI then
        local unitId = c:GetAffectedRaidUnitId(q:RemoveRealmFromName(target));
        if unitId then
            for groupId = 1, 8 do
                for subGroupId = 1, 5 do
                    local frameName = "ElvUF_Raid20Group" .. groupId .. "UnitButton" .. subGroupId; -- i assume a 20 does exist as well (just tested with the 40-frames)
                    if _G[frameName] and _G[frameName].unit and _G[frameName].unit == unitId then
                        tinsert(result, frameName);
                    end

                    frameName = "ElvUF_Raid40Group" .. groupId .. "UnitButton" .. subGroupId;
                    if _G[frameName] and _G[frameName].unit and _G[frameName].unit == unitId then
                        tinsert(result, frameName);
                    end
                end
            end
        end
    end
    return result;
end

local function GetVuhdoFrameNames(target)
    local result = {};
    if VUHDO_PANEL_UNITS then
        -- kind of a brute force attempt. but the vuhdo code is way too obfuscated
        for playerId = 1, 80 do -- seems like pets increase the number of frames...
            local frameName = "Vd2H" .. playerId;
            if _G[frameName] and _G[frameName].raidid and _G[frameName].raidid == target then
                tinsert(result, frameName .. "BgBarIcBarHlBarIc");
            end
        end
    end
    return result;
end

local function GetVuhdoRaidFrameNames(target)
    local result = {};
    if VUHDO_PANEL_UNITS then
        -- kind of a brute force attempt. but the vuhdo code is way too obfuscated
        for playerId = 1, 80 do -- seems like pets increase the number of frames...
            local frameName = "Vd1H" .. playerId;
            if _G[frameName] and _G[frameName].raidid and _G[frameName].raidid == target then
                tinsert(result, frameName);
            end
        end
    end
    return result;
end

local function GetLunaPartyFrame(target)
    for playerId = 1, 5 do
        local frameName = "LUFHeaderpartyUnitButton" .. playerId;
        if _G[frameName] then
            local unitName = UnitName(_G[frameName].unit);
            if unitName == target then
                return _G[frameName].Health;
            end
        end
    end
end

local function GetLunaRaidFrameNames(target)
    local result = {};
    for groupId = 1, 8 do
        for subGroupId = 1, 5 do
            local frameName = "LUFHeaderraid" .. groupId .. "UnitButton" .. subGroupId;
            if _G[frameName] then
                local unitName = UnitName(_G[frameName].unit);
                if unitName == target then
                    tinsert(result, frameName);
                end
            end
        end
    end
    return result;
end

local function GetGrid2FrameNames(target)
    local result = {};

    -- just grid2 stuff
    target = q:RemoveRealmFromName(target);
    if strlen(target) > 6 then
        target = strsub(target, 1, 6);
    end

    if _G["Grid2LayoutFrame"] then
        for subGroupId = 1, 8 do
            for playerId = 1, 5 do
                local frameName = "Grid2LayoutHeader" .. subGroupId .. "UnitButton" .. playerId;
                -- some kind of reverse engineering. seems to be working
                if _G[frameName] and _G[frameName]["text-down"] and _G[frameName]["text-down"].Text and
                    _G[frameName]["text-down"].Text.GetText and _G[frameName]["text-down"].Text:GetText() == target then
                    tinsert(result, frameName);
                end
            end
        end
    end
    return result;
end

-- currently unused
local function GetGrid2FrameNames2(target)
    local result = {};

    if _G["Grid2LayoutFrame"] then
        local unitId = c:GetAffectedRaidUnitId(target);

        for subGroupId = 1, 8 do
            for playerId = 1, 5 do
                local frameName = "Grid2LayoutHeader" .. subGroupId .. "UnitButton" .. playerId;
                -- some kind of reverse engineering. seems to be working
                if _G[frameName] and _G[frameName]["text-down"] and _G[frameName]["text-down"].Text and
                    _G[frameName]["text-down"].Text.GetText and _G[frameName]["text-down"].Text:GetText() == target then
                    tinsert(result, frameName);
                end
            end
        end
    end
    return result;
end

function c:GetTargetFrames(target)
    local result, partyMemberId, raidId, subGroupId, playerInSubGroupId = {}, c:GetAffectedPartyMemberId(target), c:GetAffectedRaidMemberId(target);

    ----- default wow / elvUI -----
    if QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.OPT_UIATTR_ENABLED] then
        result[c.PROFILE_DEFAULT] = {};

        -- player frames
        if QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_PLAYER][c.OPT_UIATTR_ENABLED] and target ==
            GetUnitName("player") then
            result[c.PROFILE_DEFAULT][c.PROFILE_PLAYER] = {};

            -- default player frame
            AddFrameName(result[c.PROFILE_DEFAULT][c.PROFILE_PLAYER], "PlayerFrame");

            -- elvui player frame
            AddFrameName(result[c.PROFILE_DEFAULT][c.PROFILE_PLAYER], "ElvUF_Player");
        end

        -- pet frames
            if QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_PET][c.OPT_UIATTR_ENABLED] and target == UnitName("playerpet") then
            result[c.PROFILE_DEFAULT][c.PROFILE_PET] = {};

            -- default player frame
            AddFrameName(result[c.PROFILE_DEFAULT][c.PROFILE_PET], "PetFrame");

            -- elvui player frame
            AddFrameName(result[c.PROFILE_DEFAULT][c.PROFILE_PET], "ElvUF_Pet");
        end

        -- target frames
        if QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_TARGET][c.OPT_UIATTR_ENABLED] and
            q:RemoveRealmFromName(target) == q:RemoveRealmFromName(GetUnitName("target")) then
            result[c.PROFILE_DEFAULT][c.PROFILE_TARGET] = {};
            -- default target frame
            AddFrameName(result[c.PROFILE_DEFAULT][c.PROFILE_TARGET], "TargetFrame");

            -- elvui target frame
            AddFrameName(result[c.PROFILE_DEFAULT][c.PROFILE_TARGET], "ElvUF_Target");
        end

        -- target of target (tot) frame
        if QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_TARGETTARGET][c.OPT_UIATTR_ENABLED] and
            q:RemoveRealmFromName(target) == q:RemoveRealmFromName(GetUnitName("targettarget")) then
            result[c.PROFILE_DEFAULT][c.PROFILE_TARGETTARGET] = {};

            --default
            AddFrameName(result[c.PROFILE_DEFAULT][c.PROFILE_TARGETTARGET], "TargetFrameToT");

            --elvui
            AddFrameName(result[c.PROFILE_DEFAULT][c.PROFILE_TARGETTARGET], "ElvUF_TargetTarget");
        end

        -- party frames
        if QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_PARTY][c.OPT_UIATTR_ENABLED] then
            result[c.PROFILE_DEFAULT][c.PROFILE_PARTY] = {};

            -- default party frame
            if partyMemberId then
                AddFrameName(result[c.PROFILE_DEFAULT][c.PROFILE_PARTY], "PartyMemberFrame" .. partyMemberId);
            end

            -- elvUI party frame
            AddFrameNames(result[c.PROFILE_DEFAULT][c.PROFILE_PARTY], GetElvUiPartyFrameNames(target));
        end

        -- focus frame
        if QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_FOCUS][c.OPT_UIATTR_ENABLED] and
            q:RemoveRealmFromName(target) == q:RemoveRealmFromName(UnitName("focus")) then
            result[c.PROFILE_DEFAULT][c.PROFILE_FOCUS] = {};

            -- default
            AddFrameName(result[c.PROFILE_DEFAULT][c.PROFILE_FOCUS], "FocusFrame");

            -- elvUI
            AddFrameName(result[c.PROFILE_DEFAULT][c.PROFILE_FOCUS], "ElvUF_Focus");
        end

        -- focus target frame
        if QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_FOCUSTARGET][c.OPT_UIATTR_ENABLED] and
            q:RemoveRealmFromName(target) == q:RemoveRealmFromName(UnitName("focustarget")) then
            result[c.PROFILE_DEFAULT][c.PROFILE_FOCUSTARGET] = {};
            AddFrameName(result[c.PROFILE_DEFAULT][c.PROFILE_FOCUSTARGET], "FocusFrameToT");
        end

        -- nameplates
        --if QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_NAMEPLATES][c.OPT_UIATTR_ENABLED] then
        --    result[c.PROFILE_DEFAULT][c.PROFILE_NAMEPLATES] = {};
        --    local namePlate = q:GetPlayerNamePlate(target);
        --    if namePlate then
        --        AddFrame(result[c.PROFILE_DEFAULT][c.PROFILE_NAMEPLATES], namePlate);
        --    end
        --end
    end

    -- raid frames
    if QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_RAID_COMPACT][c.OPT_UIATTR_ENABLED] or
        QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_RAID_PULLOUT][c.OPT_UIATTR_ENABLED] then
        if QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_RAID_COMPACT][c.OPT_UIATTR_ENABLED] then
            result[c.PROFILE_DEFAULT][c.PROFILE_RAID_COMPACT] = {};

            if _G["CompactRaidFrameContainer"]:IsVisible() then
                -- default compactraidgroup / compactraidframe / compact party frames
                AddFrames(result[c.PROFILE_DEFAULT][c.PROFILE_RAID_COMPACT], GetRaidFrames(target));
            end

            -- ElvUI (overrides default raid frames)
            AddFrameNames(result[c.PROFILE_DEFAULT][c.PROFILE_RAID_COMPACT], GetElvUiRaidFrameNames(target));
        end

        if QOL_OPTIONS[c.name][c.PROFILE_DEFAULT][c.PROFILE_RAID_PULLOUT][c.OPT_UIATTR_ENABLED] and subGroupId and
            playerInSubGroupId then
            result[c.PROFILE_DEFAULT][c.PROFILE_RAID_PULLOUT] = {};

            -- default raid pull out group
            AddFrame(result[c.PROFILE_DEFAULT][c.PROFILE_RAID_PULLOUT], GetRaidPulloutFrame(target));
        end
    end

    ----- addons -----

    -- luna
    if c:GetProfile(c.PROFILE_ADDON_LUNA) and QOL_OPTIONS[c.name][c.PROFILE_ADDON_LUNA][c.OPT_UIATTR_ENABLED] then
        result[c.PROFILE_ADDON_LUNA] = {};

        -- player frame
        if QOL_OPTIONS[c.name][c.PROFILE_ADDON_LUNA][c.PROFILE_PLAYER][c.OPT_UIATTR_ENABLED] and target == UnitName("player") then
            result[c.PROFILE_ADDON_LUNA][c.PROFILE_PLAYER] = {};
            if _G["LUFUnitplayer"] and _G["LUFUnitplayer"]:IsVisible() then
                AddFrame(result[c.PROFILE_ADDON_LUNA][c.PROFILE_PLAYER], _G["LUFUnitplayer"].Health);
            end
        end

        -- pet frame
        if QOL_OPTIONS[c.name][c.PROFILE_ADDON_LUNA][c.PROFILE_PET][c.OPT_UIATTR_ENABLED] and target == UnitName("playerpet") then
            result[c.PROFILE_ADDON_LUNA][c.PROFILE_PET] = {};
            if _G["LUFUnitpet"] and _G["LUFUnitpet"]:IsVisible() then
                AddFrame(result[c.PROFILE_ADDON_LUNA][c.PROFILE_PET], _G["LUFUnitpet"].Health);
            end
        end

        -- target frame
        if QOL_OPTIONS[c.name][c.PROFILE_ADDON_LUNA][c.PROFILE_TARGET][c.OPT_UIATTR_ENABLED] and target == UnitName("target") then
            result[c.PROFILE_ADDON_LUNA][c.PROFILE_TARGET] = {};
            if _G["LUFUnittarget"] and _G["LUFUnittarget"]:IsVisible() then
                AddFrame(result[c.PROFILE_ADDON_LUNA][c.PROFILE_TARGET], _G["LUFUnittarget"].Health);
            end
        end

        -- targettarget
        if QOL_OPTIONS[c.name][c.PROFILE_ADDON_LUNA][c.PROFILE_TARGETTARGET][c.OPT_UIATTR_ENABLED] and target == UnitName("targettarget") then
            result[c.PROFILE_ADDON_LUNA][c.PROFILE_TARGETTARGET] = {};
            if _G["LUFUnittargettarget"] and _G["LUFUnittargettarget"]:IsVisible() then
                AddFrame(result[c.PROFILE_ADDON_LUNA][c.PROFILE_TARGETTARGET], _G["LUFUnittargettarget"].Health);
            end
        end

        -- targettargettarget
        if QOL_OPTIONS[c.name][c.PROFILE_ADDON_LUNA][c.PROFILE_TARGETTARGETTARGET][c.OPT_UIATTR_ENABLED] and target == UnitName("targettargettarget") then
            result[c.PROFILE_ADDON_LUNA][c.PROFILE_TARGETTARGETTARGET] = {};
            if _G["LUFUnittargettargettarget"] and _G["LUFUnittargettargettarget"]:IsVisible() then
                AddFrame(result[c.PROFILE_ADDON_LUNA][c.PROFILE_TARGETTARGETTARGET], _G["LUFUnittargettargettarget"].Health);
            end
        end

        -- focus
        if QOL_OPTIONS[c.name][c.PROFILE_ADDON_LUNA][c.PROFILE_FOCUS][c.OPT_UIATTR_ENABLED] and target == UnitName("focus") then
            result[c.PROFILE_ADDON_LUNA][c.PROFILE_FOCUS] = {};
            if _G["LUFUnitfocus"] and _G["LUFUnitfocus"]:IsVisible() then
                AddFrame(result[c.PROFILE_ADDON_LUNA][c.PROFILE_FOCUS], _G["LUFUnitfocus"].Health);
            end
        end

        -- party frame
        if QOL_OPTIONS[c.name][c.PROFILE_ADDON_LUNA][c.PROFILE_PARTY][c.OPT_UIATTR_ENABLED] then
            result[c.PROFILE_ADDON_LUNA][c.PROFILE_PARTY] = {GetLunaPartyFrame(target)};
        end

        -- raid
        if QOL_OPTIONS[c.name][c.PROFILE_ADDON_LUNA][c.PROFILE_RAID_COMPACT][c.OPT_UIATTR_ENABLED] then
            result[c.PROFILE_ADDON_LUNA][c.PROFILE_RAID_COMPACT] = {};
            AddFrameNames(result[c.PROFILE_ADDON_LUNA][c.PROFILE_RAID_COMPACT], GetLunaRaidFrameNames(target));
        end
    end

    -- grid2
    if c:GetProfile(c.PROFILE_ADDON_GRID2) and subGroupId and playerInSubGroupId and
        QOL_OPTIONS[c.name][c.PROFILE_ADDON_GRID2][c.OPT_UIATTR_ENABLED] and
        QOL_OPTIONS[c.name][c.PROFILE_ADDON_GRID2][c.PROFILE_RAID_COMPACT][c.OPT_UIATTR_ENABLED] then
        result[c.PROFILE_ADDON_GRID2] = {};
        result[c.PROFILE_ADDON_GRID2][c.PROFILE_RAID_COMPACT] = {};
        AddFrameNames(result[c.PROFILE_ADDON_GRID2][c.PROFILE_RAID_COMPACT], GetGrid2FrameNames(target));
    end

    -- vuhdo
    if c:GetProfile(c.PROFILE_ADDON_VUHDO) and QOL_OPTIONS[c.name][c.PROFILE_ADDON_VUHDO][c.OPT_UIATTR_ENABLED] then
        result[c.PROFILE_ADDON_VUHDO] = {};

        -- player
        if (target == GetUnitName("player") or target == "player") and
            QOL_OPTIONS[c.name][c.PROFILE_ADDON_VUHDO][c.PROFILE_PLAYER][c.OPT_UIATTR_ENABLED] then
            result[c.PROFILE_ADDON_VUHDO][c.PROFILE_PLAYER] = {};
            AddFrameNames(result[c.PROFILE_ADDON_VUHDO][c.PROFILE_PLAYER], GetVuhdoFrameNames("player"));
        end

        -- target
        if (target == UnitName("target") or target == "target") and
            QOL_OPTIONS[c.name][c.PROFILE_ADDON_VUHDO][c.PROFILE_TARGET][c.OPT_UIATTR_ENABLED] then
            result[c.PROFILE_ADDON_VUHDO][c.PROFILE_TARGET] = {};
            AddFrameNames(result[c.PROFILE_ADDON_VUHDO][c.PROFILE_TARGET], GetVuhdoFrameNames("target"));
        end

        if QOL_OPTIONS[c.name][c.PROFILE_ADDON_VUHDO][c.PROFILE_RAID_COMPACT][c.OPT_UIATTR_ENABLED] then
            result[c.PROFILE_ADDON_VUHDO][c.PROFILE_RAID_COMPACT] = {};
            -- raid frames
            AddFrameNames(result[c.PROFILE_ADDON_VUHDO][c.PROFILE_RAID_COMPACT],
                GetVuhdoRaidFrameNames(c:GetAffectedRaidUnitId(target))); -- by raid
            AddFrameNames(result[c.PROFILE_ADDON_VUHDO][c.PROFILE_RAID_COMPACT],
                GetVuhdoRaidFrameNames(c:GetAffectedPartyUnitId(target))); -- by party
        end
    end

    return result;
end
